package factory.clients;

import java.rmi.RemoteException;
import javax.ejb.*;
import javax.naming.*;
import javax.rmi.PortableRemoteObject;
import java.util.Properties;
import java.io.*;

import factory.manufacture.Manufacture;
import factory.manufacture.ManufactureHome;
import factory.manage_orders.OpenOrderView;

public class BeginManufacture
{
  private static final String MANUFACTURE_CELL = "Station1";
  private static final String FILE_NAME = "C:/current_product.ser";

  public static void main(String[] args)
  {
    try
    {
      ContextPropertiesFactory factory = new ContextPropertiesFactory();
      Properties prop = factory.getInitialContextProperties();
      InitialContext initial = new InitialContext(prop);

      Object homeObject = initial.lookup("Manufacture");

      ManufactureHome home = (ManufactureHome)
        PortableRemoteObject.narrow(homeObject, ManufactureHome.class);

      Manufacture manufacture = home.create(MANUFACTURE_CELL);
      OpenOrderView[] openOrders = manufacture.getOpenOrders();

      if(openOrders.length == 0)
      {
        System.out.println("Nothing to make; go home.");
        return;
      }

      System.out.println("Selecting from the following open orders:");
      for(int iter = 0; iter < openOrders.length; iter++)
      {
        OpenOrderView openOrder = openOrders[iter];
        System.out.println("Sales Division: " + openOrder.salesDivision +
                           "; Order #: " + openOrder.orderNumber + 
						   "; Product: " + openOrder.product + "Date due: " 
                           + openOrder.dateDue );
      }

      // Get the first open order
      for (int iterFind = 0; iterFind < openOrders.length; iterFind++)
      {
        try
        {
          OpenOrderView openOrder = openOrders[iterFind];
          manufacture.selectForManufacture(
                     openOrder.salesDivision, openOrder.orderNumber);
          Handle handle = manufacture.getHandle();
          FileOutputStream file_out = new FileOutputStream(FILE_NAME);
          ObjectOutputStream out = new ObjectOutputStream(file_out);
          out.writeObject(handle);
          System.out.println("Written object for next stage.");
          break;
        }
        catch(factory.manufacture.BadStatusException bse)
        {
          // Someone grabbed it before we did
          bse.printStackTrace();
        }
      }

    }
    catch (FileNotFoundException fnfe)
    {
      fnfe.printStackTrace();
    }
    catch (RemoteException re)
    {
      re.printStackTrace();
    }
    catch (IOException ioe)
    {
      ioe.printStackTrace();
    }
    catch (factory.manage_orders.NoSuchOrderException nsoe)
    {
      nsoe.printStackTrace();
    }
    catch (NamingException ne)
    {
      ne.printStackTrace();
    }
    catch (CreateException ce)
    {
      ce.printStackTrace();
    }
  }
}
