package factory.manage_orders;

import javax.ejb.EJBException;
import factory.product.Product;
import factory.product.ProductHome;
import factory.product.RoutingInstruction;
import factory.product.NoSuchRoutingInstruction;

public class ProductCreationHelper
{
  ProductHome productHome;

  public ProductCreationHelper(ProductHome productHome)
  {
    this.productHome = productHome;
  }

  public void createAll()
  {
    try
    {
      createDesk();
      createChair();
      createLamp();
    }
    catch(Exception e)
    {
      e.printStackTrace();
      throw new EJBException(e);
    }
  }

  public void createDesk() throws Exception
  {
    RoutingInstruction compress = new RoutingInstruction(5, 
                                      "Compress the wood.");
    RoutingInstruction stain = new RoutingInstruction(10, 
                                   "Stain the wood.");
    RoutingInstruction assemble = new RoutingInstruction(15, 
                                      "Assemble the desk.");
    RoutingInstruction[] routings = new RoutingInstruction[]{
                                        compress, stain, assemble};

    productHome.create("DESK01", "Compressed Wood Desk", routings);
  }

  public void createChair() throws Exception
  {
    RoutingInstruction extrude = new RoutingInstruction(5, 
                                     "Extrude plastic.");
    RoutingInstruction glue = new RoutingInstruction(10, 
                                  "Glue together.");
    RoutingInstruction paint = new RoutingInstruction(15, 
                                   "Spraypaint.");
    RoutingInstruction[] routings = new RoutingInstruction[]{
                                        extrude, glue, paint};

    productHome.create("CHAIR01", "Quality Plastic Chair", routings);
  }

  public void createLamp() throws Exception
  {
    RoutingInstruction getBulb = new RoutingInstruction(5, 
                                     "Get bulb from inventory.");
    RoutingInstruction getLamp = new RoutingInstruction(10, 
                                     "Get lamp from inventory.");
    RoutingInstruction screwTogether = new RoutingInstruction(15, 
                                           "Screw together.");
    RoutingInstruction frayCord = new RoutingInstruction(20, 
                                      "Pre-fray the cord");
    RoutingInstruction[] routings = new RoutingInstruction[]{
                                        getBulb, getLamp, screwTogether, 
                                        frayCord};

    Product lamp = productHome.create("LAMP01", "Custom Made Lamp", 
                                       routings);
    try
    {
      lamp.deleteRoutingInstruction(21);
    }
    catch(NoSuchRoutingInstruction nsri)
    {
      lamp.deleteRoutingInstruction(20);
    }

  }

}
