package tv; // poderia estar num package mais genrico

import java.awt.*;
import java.beans.*;
import java.io.*;

/**
 * Um boto que acende quando est "on"
 */
public class BotaoOnOff extends Button
                        implements Serializable, PropertyChangeListener {

  protected boolean   on; // estado do boto
  protected PropertyChangeSupport pcs;

  // Construtor
  public BotaoOnOff() {
    // Cada bean tem um construtor default ("no-arg")
    super("Liga/Desliga");
    pcs = new PropertyChangeSupport(this);
    on = false;
  }

  // Mtodo getter para propriedade "on"
  public boolean isOn() {
    return on;
  }

  // Mtodo setter para propriedade "on"
  public void setOn(boolean on) {
    // Dispara a mudana para os listeners, se houver
    pcs.firePropertyChange("on", new Boolean(this.on), new Boolean(on));
    this.on = on;
    mudaVisual();
  }

  // Muda o visual da "lmpada on/off"
  private void mudaVisual() {
    setBackground(isOn() ? Color.red : Color.gray);
  }

  // Se este bean for listener da propriedade "on" de outro bean,
  // entao precisamos deste mtodo
  public void propertyChange(PropertyChangeEvent pce) {
    String propriedadeMudada = pce.getPropertyName();
    Object novoValor = pce.getNewValue();
    if(propriedadeMudada.equals("on")) {
      this.setOn(((Boolean)novoValor).booleanValue());
    }
  }

  // Mtodo de cadastro de listeners
  // usa delegao para implementar
  public void addPropertyChangeListener(PropertyChangeListener l) {
    pcs.addPropertyChangeListener(l);
  }

  // Mtodo de remoo de listeners
  // usa delegao para implementar
  public void removePropertyChangeListener(PropertyChangeListener l) {
    pcs.removePropertyChangeListener(l);
  }
}

