package tv;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.io.*;

/**
 * Um keypad
 */
public class KeyPad extends Container implements ActionListener {
  protected int   valor = 0;      // propriedade do bean
  protected int   maxDgitos = 3; // propriedade do bean: nmero max de dgitos
  private String  valorDisplay;   // valor mostrado no display antes de dar ENTER
  protected PropertyChangeSupport pcs;
  private Button botoEnter = new Button("ENTER");
  private Label display = new Label("");

  // Construtor
  public KeyPad() {
    pcs = new PropertyChangeSupport(this);
    Panel p = new Panel(); // para os botes de 0 a 9
    p.setLayout(new GridLayout(2,5));
    setLayout(new BorderLayout()); // para conter o keypad inteiro

    // cria os botes
    for(int i = 0; i < 10; i++) {
      Button b = new Button(Integer.toString(i));
      p.add(b);
      b.addActionListener(this);
    }
    botoEnter.addActionListener(this);
    add("South", botoEnter);
    add("Center", p);
    add("North", display);
    display.setAlignment(Label.CENTER);
    valorDisplay = "";
  }

  // getter do valor
  public int getValor() {
    return valor;
  }

  // setter do valor
  public void setValor(int valor) {
    pcs.firePropertyChange("valor", new Integer(this.valor),
                                    new Integer(valor));
    this.valor = valor;
    valorDisplay = "";
    display.setText(valorDisplay);
  }

  // getter de maxDgitos
  public int getMaxDgitos() {
    return maxDgitos;
  }

  // setter de maxDgitos
  public void setMaxDgitos(int maxDgitos) {
    this.maxDgitos = maxDgitos;
  }

  // Mudana do display
  public void mudaDisplay(String digito) {
    valorDisplay += digito;
    // s permite maxDgitos digitos de display
    if(valorDisplay.length() > maxDgitos) {
      valorDisplay = digito;
    }
    display.setText(valorDisplay);
  }

  // cadastro para mudana de propriedade
  public void addPropertyChangeListener(PropertyChangeListener l) {
    pcs.addPropertyChangeListener(l);
  }

  // remoo de cadastro para mudana de propriedade
  public void removePropertyChangeListener(PropertyChangeListener l) {
    pcs.removePropertyChangeListener(l);
  }

  // trata o evento de clique de qualquer boto
  public void actionPerformed(ActionEvent ae) {
    if(ae.getSource().equals(botoEnter)) {
      try {
        setValor(Integer.parseInt(valorDisplay));
      } catch(NumberFormatException nfe) {
        // "no deveria" acontecer
        nfe.printStackTrace();
        return;
      }
    } else {
      mudaDisplay(ae.getActionCommand());
    }
  }
}

