/*
 * Desenvolvido para a disciplina Programacao 1
 * Curso de Bacharelado em Cincia da Computao
 * Departamento de Sistemas e Computao
 * Universidade Federal da Paraba
 *
 * Copyright (C) 1999 Universidade Federal da Paraba.
 * No redistribuir sem permisso.
 */
package p1.aplic.banco;
import java.io.*;

/**
 * Classe abstrata representando uma moeda genrica com centavos.
 * O motivo da existncia dessa classe  de permitir a manipulao
 * de valores financeiros sem se preocupar com fraes de centavos.
 * Classes clientes podem usar double para manipular valores mas,
 * ao passar tais valores para uma Moeda, as fraes de centavos
 * somem.
 * <P>A classe  abstrata porque no sabemos o nome da moeda e no temos portanto
 * um mtodo toString().
 *
 * @author Jacques Philippe Sauv, jacques@dsc.ufpb.br
 * @version 1.1 (adicionei setCentavos)
 * <br>
 * Copyright (C) 1999, 2000 Universidade Federal da Paraba.
 */

public abstract class Moeda implements Serializable {
  static final long serialVersionUID = -51315915643707243L;
  private long  centavos;

  /**
   * Controi um valor 0,0 como Moeda.
   */
  public Moeda() {
    this(0.0);
  }

  /**
   * Controi um valor como Moeda.
   * @param valor O valor a representar.
   */
  public Moeda(double valor) {
    setValor(valor);
  }

  /**
   * Recupera o valor como double.
   */
  public double getValor() {
    return centavos / 100.0;
  }

  /**
   * Ajusta o valor a ser representado como moeda.
   * @param valor O valor a representar como moeda.
   */
  public void setValor(double valor) {
    // Vai perder fraes de centavos aqui
    // Ajustar 0.5 centavos  para arredondar ao
    // centavo mais prximo
    if(valor >= 0) {
      this.centavos = (long)(valor*100.0 + 0.5);
    } else {
      this.centavos = (long)(valor*100.0 - 0.5);
    }
  }

  /**
   * Compara igualdade de duas moedas.
   * @param moeda O outro valor a comparar.
   */
  public boolean equals(Object outroObjeto) {
    if(!(outroObjeto instanceof Moeda)) {
      return false;
    }
    return getValor() == ((Moeda)outroObjeto).getValor();
  }

  /**
   * Compara dois valores de moeda.
   * @param outra A outra moeda a comparar.
   * @return 0 se a moeda for igual  outra moeda; -1 se ela for menor e +1 se for maior.
   */
   public int compareTo(Moeda outra) {
     long diferena = getCentavos() - outra.getCentavos();
     return diferena == 0 ? 0 : (diferena < 0 ? -1 : 1);
   }

  /**
   * Representa o valor da moeda como string.
   */
  public abstract String toString();

  protected long getCentavos() {
    return centavos;
  }
  protected void setCentavos(long centavos) {
    this.centavos = centavos;
  }
}