/*
 * Desenvolvido para a disciplina Programacao 1
 * Curso de Bacharelado em Cincia da Computao
 * Departamento de Sistemas e Computao
 * Universidade Federal da Paraba
 *
 * Copyright (C) 1999 Universidade Federal da Paraba.
 * No redistribuir sem permisso.
 */
package p1.aplic.banco;

import java.io.*;
import java.util.*;
import p1.aplic.geral.*;

/**
 * Uma transao bancria. Uma transao envolve a transferncia de valores
 * entre contas. Toda a movimentao bancria envolve transaes.
 * Quando h, aparentemente, uma nica conta envolvida (depsito ou saque),
 * a segunda conta envolvida  a Conta Caixa.
 *
 * @see p1.aplic.banco.ContaCaixa
 *
 * @author   Jacques Philippe Sauv, jacques@dsc.ufpb.br
 * @version 1.1
 * <br>
 * Copyright (C) 1999 Universidade Federal da Paraba.
 */
public class Transacao implements Serializable {
  static final long serialVersionUID = 6150522172556957388L;
	private Data	data;
	private Conta	contaDbito;
	private Conta	contaCrdito;
	private Moeda	valor;
	private String	descrio;

	// Construtores
    /**
     * Constroi uma transao.
     * @param data A data da transao.
     * @param contaDbito A conta que foi debitado.
     * @param contaCrdito A conta que foi creditado.
     * @param valor O valor da transao.
     * @param descrio A descrio da transao.
     */
	public Transacao(Data data, Conta contaDbito, Conta contaCrdito, Moeda valor, String descrio) {
		this.data = data;
		this.contaDbito = contaDbito;
		this.contaCrdito = contaCrdito;
		this.valor = valor;
		this.descrio = descrio;
	}


	/**
	 * Recupera a data da transao.
	 * @return A data da transao.
	 */
	public Data getData() {
		return data;
	}
   
	/**
	 * Recupera a conta dbito.
	 * @return A conta dbito.
	 */
	public Conta getContaDbito() {
		return contaDbito;
	}

	/**
	 * Recupera a conta crdito.
	 * @return A conta crdito.
	 */
	public Conta getContaCrdito() {
		return contaCrdito;
	}

	/**
	 * Recupera o valor da transao.
	 * @return O valor da transao como double.
	 */
	public double getValor() {
		return valor.getValor();
	}

	/**
	 * Recupera o valor da transao.
	 * @return O valor da transao como Moeda.
	 */
	public Moeda getValorMonetrio() {
		return valor;
	}

	/**
	 * Recupera a descrio da transao.
	 * @return A descrio da transao.
	 */
	public String getDescrio() {
		return descrio;
	}

	/**
	 * Transforma os dados da transao em um String.
	 * @return O string descrevendo a transao.
	 */
	public String toString() {
		return "Transacao data " + getData().DDMMAAAA()
			+ ", debito " + getContaDbito().getNmero()
			+ ", credito " + getContaCrdito().getNmero()
			+ ", valor " + getValorMonetrio()
			+ ", descricao [" + getDescrio();
	}
}
