/*
 * Desenvolvido para a disciplina Programacao 1
 * Curso de Bacharelado em Cincia da Computao
 * Departamento de Sistemas e Computao
 * Universidade Federal da Paraba
 *
 * Copyright (C) 1999 Universidade Federal da Paraba.
 * No redistribuir sem permisso.
 */

package p1.aplic.cartas;

import java.util.*;
import java.lang.Math.*;

/**
 * Como a classe Baralho, mas lana exceo quando pega uma carta num baralho vazio.
 * No usa herana para poder usar o exemplo antes do aluno ver herana.
 *
 * @author Jacques Philippe Sauv, jacques@dsc.ufpb.br
 * @version 1.1
 * <br>
 * Copyright (C) 1999 Universidade Federal da Paraba.
 */
public class BaralhoSeguro {
  /**
   * O baralho  armazenado aqui.  protected porque alguns baralhos
   * subclasses dessa classe podero talvez ter que mexer diretamente aqui
   * para construir baralhos especiais.
   */
  protected Vector baralho;

  /**
   * Construtor de um baralho comum.
   */
  public BaralhoSeguro() {
    // Usa um Vector para ter um iterador facilmente
    baralho = new Vector();
    // enche o baralho
    for(int valor = menorValor(); valor <= maiorValor(); valor++) {
      for(int naipe = primeiroNaipe(); naipe <= ltimoNaipe(); naipe++) {
        // chama criaCarta e no "new" para poder fazer override
        // de criaCarta em baralhos de subclasses e criar classes diferentes.
        baralho.add(criaCarta(valor, naipe));
      }
    }
  }

  /**
   * Cria uma carta para este baralho.
   * @param valor O valor da carta a criar.
   * @param naipe O naipe da carta a criar.
   * @return A carta criada.
   */
  protected Carta criaCarta(int valor, int naipe) {
    return new Carta(valor, naipe);
  }

  /**
   * Recupera o valor da menor carta possvel deste baralho.
   *  possvel fazer um lao de menorValor() at maiorValor() para varrer
   * todos os valores possveis de cartas.
   * @return O menor valor.
   */
  public int menorValor() {
    return Carta.menorValor();
  }

  /**
   * Recupera o valor da maior carta possvel deste baralho.
   *  possvel fazer um lao de menorValor() at maiorValor() para varrer
   * todos os valores possveis de cartas.
   * @return O maior valor.
   */
  public int maiorValor() {
    return Carta.maiorValor();
  }

  /**
   * Recupera o "primeiro naipe" das cartas que podem estar no baralho.
   * Ser "primeiro naipe" no significa muita coisa, j que naipes no tem valor
   * (um naipe no  menor ou maior que o outro).
   * Fala-se de "primeiro naipe" e "ltimo naipe" para poder
   * fazer um lao de primeiroNaipe() at ltimoNaipe() para varrer
   * todos os naipes possveis de cartas.
   * @return O primeiro naipe.
   */
  public int primeiroNaipe() {
    return Carta.primeiroNaipe();
  }

  /**
   * Recupera o "ltimo naipe" das cartas que podem estar no baralho.
   * Ser "ltimo naipe" no significa muita coisa, j que naipes no tem valor
   * (um naipe no  menor ou maior que o outro).
   * Fala-se de "primeiro naipe" e "ltimo naipe" para poder
   * fazer um lao de primeiroNaipe() at ltimoNaipe() para varrer
   * todos os naipes possveis de cartas.
   * @return O primeiro naipe.
   */
  public int ltimoNaipe() {
    return Carta.ltimoNaipe();
  }

  /**
   * Recupera o nmero de cartas atualmente no baralho.
   * @return O nmero de cartas no baralho.
   */
  public int nmeroDeCartas() {
    return baralho.size();
  }
  
  /**
   * Recupera um iterador para poder varrer todas as cartas do baralho.
   * @return Um iterador do baralho.
   */
  public Iterator iterator() {
    return baralho.iterator();
  }

  /**
   * Baralha (traa) o baralho.
   */
  public void baralhar() {
    int posio;
    for(posio = 0; posio < nmeroDeCartas() - 1; posio++) {
      // escolhe uma posio aleatria entre posio e nmeroDeCartas()-1
      int posAleatria = posio + (int)((nmeroDeCartas() - posio) * Math.random());
      // troca as cartas em posio e posAleatria
      Carta temp = (Carta)baralho.get(posio);
      baralho.set(posio, baralho.get(posAleatria));
      baralho.set(posAleatria, temp);
    }
  }

  /**
   * Retira uma carta do topo do baralho e a retorna. A carta  removida do baralho.
   * @return A carta retirada do baralho.
   * @throws BaralhoVazioException Quando o baralho est vazio.
   */
  public Carta pegaCarta() throws BaralhoVazioException {
    if(nmeroDeCartas() == 0) {
      throw new BaralhoVazioException("Baralho esta vazio");
    }
    return (Carta)baralho.remove(nmeroDeCartas()-1);
  }
}

