/*
 * Desenvolvido para a disciplina Programacao 1
 * Curso de Bacharelado em Cincia da Computao
 * Departamento de Sistemas e Computao
 * Universidade Federal da Paraba
 *
 * Copyright (C) 1999 Universidade Federal da Paraba.
 * No redistribuir sem permisso.
 */

/* deveria ter uma superclasse para fatorar o que h de comum em Baralho
 * e Mao, mas no queremos mostrar isso aos principiantes ainda
 */

package p1.aplic.cartas;

import java.util.*;

/**
 * Uma mo de cartas.
 *
 * @author Jacques Philippe Sauv, jacques@dsc.ufpb.br
 * @version 1.1
 * <br>
 * Copyright (C) 1999 Universidade Federal da Paraba.
 */
public class Mao {
	private Vector cartas;

    /**
     * Construtor de uma mo (vazia) de cartas.
     */
	public Mao() {
        // Usa um Vector para ter um iterador facilmente
		cartas = new Vector();
    }

    /**
     * Recupera o nmero de cartas na mo.
     * @return O nmero de cartas na mo.
     */
    public int nmeroDeCartas() {
        return cartas.size();
    }

    /**
     * Recupera um iterador para poder varrer todas as cartas da mo.
     * @return Um iterador da mo.
     */
    public Iterator iterator() {
        return cartas.iterator();
    }

    /**
     * Calcula o valor da mo. O valor da mo vai depender normalmente
     * de todas as cartas que tem nela. O valor absoluto da mo poder
     * ter algum significado ou no, dependendo do jogo em particular.
     * O mais importante  que duas mos podem ser comparadas de acordo com este valor.
     * @return O valor da mo.
     */
    public int valor() {
        Iterator it = iterator();
        int     resultado = 0;
        while(it.hasNext()) {
            resultado += ((Carta)it.next()).getValor();
        }
        return resultado;
    }

    /**
     * Adiciona uma carta  mo.
     * @param carta A carta a adicionar.
     */
	public void adicionar(Carta carta) {
		cartas.add(carta);
	}

    /**
     * Recupera a carta que est numa determinada posio na mo.
     * A carta mais  esquerda  de posio 0, e esta posio
     * cresce para as cartas  direita.
     * A carta permanece na mo aps este mtodo.
     * @param posio A posio da carta desejada.
     * @return A carta na posio desejada.
     */
    public Carta cartaEm(int posio) {
        return posio < nmeroDeCartas() ? (Carta)cartas.get(posio) : null;
    }

    /**
     * Recupera a carta que est "no topo" da mo (isto , a carta mais  direita).
     * @return A carta desejada.
     */
    public Carta cartaNoTopo() {
        return cartaEm(nmeroDeCartas()-1);
    }

    /**
     * Remove a carta que est numa determinada posio na mo.
     * A carta mais  esquerda  de posio 0, e esta posio
     * cresce para as cartas  direita.
     * @param posio A posio da carta desejada.
     * @return A carta desejada.
     */
    public Carta remover(int i) {
        return i < nmeroDeCartas() ? (Carta)cartas.remove(i) : null;
    }

    /**
     * Remove uma determinada carta da mo.
     * @param carta A carta a ser removida da mo.
     * @return true se a carta foi removida, false caso contrrio.
     */
    public boolean remover(Carta carta) {
        return cartas.remove(carta);
    }

    /**
     * Troca duas cartas da mo, dadas duas posies na mo.
     * A carta mais  esquerda  de posio 0, e esta posio
     * cresce para as cartas  direita.
     * @param i A posio da primeira carta a trocar.
     * @param j A posio da segunda carta a trocar.
     */
    public void trocar(int i, int j) {
        int n = nmeroDeCartas();
        if(i < n && j < n) {
            Carta temp = (Carta)cartas.get(i);
			cartas.set(i, cartas.get(j));
			cartas.set(j, temp);
        }
    }

    /**
     * Troca duas cartas da mo.
     * @param c1 A primeira carta a trocar.
     * @param c2 A segunda carta a trocar.
     */
    public void trocar(Carta c1, Carta c2) {
        int i1 = cartas.indexOf(c1);
        if(i1 >= 0) {
            int i2 = cartas.indexOf(c2);
            if(i2 >= 0) {
                trocar(i1, i2);
            }
        }
    }

    /**
     * Compare esta mo a outra de acordo com seu respectivo valor.
     * @param outra A outra mo a comparar.
     * @return 0 se as mos forem de mesmo valor;
     * um valor < 0 se esta mo for de valor menor; e
     * um valor > 0 se esta mo for de valor menor.
     */
    public int compareTo(Mao outra) {
        return valor() - outra.valor();
    }

    /**
     * Formata a mo como string.
     * @return Um string que represente a mo de cartas.
     */
    public String toString() {
        Iterator it = iterator();
        String resultado = "";
        String separador = "";
        while(it.hasNext()) {
            resultado += separador + (Carta)it.next();
            separador = ", ";
        }
        return resultado;
    }
}
